@@ -13,13 +13,13 @@
 
 #include <string>
 #include <string_view>
+#include <vector>
 #include <map>
 
 #include "ILexer.h"
 #include "Scintilla.h"
 #include "SciLexer.h"
 
-#include "StringCopy.h"
 #include "WordList.h"
 #include "LexAccessor.h"
 #include "Accessor.h"
@@ -27,6 +27,7 @@
 #include "CharacterSet.h"
 #include "LexerModule.h"
 #include "OptionSet.h"
+#include "SubStyles.h"
 #include "DefaultLexer.h"
 
 using namespace Scintilla;
@@ -38,15 +39,16 @@ namespace {
 // return 1 for [[ or ]], returns >=2 for [=[ or ]=] and so on.
 // The maximum number of '=' characters allowed is 254.
 int LongDelimCheck(StyleContext &sc) {
+	constexpr int maximumEqualCharacters = 254;
 	int sep = 1;
-	while (sc.GetRelative(sep) == '=' && sep < 0xFF)
+	while (sc.GetRelative(sep) == '=' && sep <= maximumEqualCharacters)
 		sep++;
 	if (sc.GetRelative(sep) == sc.ch)
 		return sep;
 	return 0;
 }
 
-const char * const luaWordListDesc[] = {
+const char *const luaWordListDesc[] = {
 	"Keywords",
 	"Basic functions",
 	"String, (table) & math functions",
@@ -55,9 +57,11 @@ const char * const luaWordListDesc[] = {
 	"user2",
 	"user3",
 	"user4",
-    nullptr
+	nullptr
 };
 
+const char styleSubable[] = { SCE_LUA_IDENTIFIER, 0 };
+
 const LexicalClass lexicalClasses[] = {
 	// Lexer Lua SCLEX_LUA SCE_LUA_:
 	0, "SCE_LUA_DEFAULT", "default", "White space: Visible only in View Whitespace mode (or if it has a back colour)",
@@ -107,15 +111,20 @@ class LexerLua : public DefaultLexer {
 	WordList keywords8;
 	OptionsLua options;
 	OptionSetLua osLua;
+	SubStyles subStyles{styleSubable};
 public:
 	explicit LexerLua() :
-		DefaultLexer("lua", SCLEX_LUA, lexicalClasses, ELEMENTS(lexicalClasses)) {
+		DefaultLexer("lua", SCLEX_LUA, lexicalClasses, std::size(lexicalClasses)) {
 	}
+	LexerLua(const LexerLua &) = delete;
+	LexerLua(LexerLua &&) = delete;
+	LexerLua &operator=(const LexerLua &) = delete;
+	LexerLua &operator=(LexerLua &&) = delete;
 	~LexerLua() override = default;
 	void SCI_METHOD Release() noexcept override {
 		delete this;
 	}
-	int SCI_METHOD Version() const noexcept override {
+	[[nodiscard]] int SCI_METHOD Version() const noexcept override {
 		return lvRelease5;
 	}
 	const char *SCI_METHOD PropertyNames() noexcept override {
@@ -138,6 +147,35 @@ public:
 	void SCI_METHOD Lex(Sci_PositionU startPos, Sci_Position length, int initStyle, IDocument *pAccess) override;
 	void SCI_METHOD Fold(Sci_PositionU startPos, Sci_Position length, int initStyle, IDocument *pAccess) override;
 
+	int SCI_METHOD AllocateSubStyles(int styleBase, int numberStyles) override {
+		return subStyles.Allocate(styleBase, numberStyles);
+	}
+	int SCI_METHOD SubStylesStart(int styleBase) override {
+		return subStyles.Start(styleBase);
+	}
+	int SCI_METHOD SubStylesLength(int styleBase) override {
+		return subStyles.Length(styleBase);
+	}
+	int SCI_METHOD StyleFromSubStyle(int subStyle) override {
+		const int styleBase = subStyles.BaseStyle(subStyle);
+		return styleBase;
+	}
+	int SCI_METHOD PrimaryStyleFromStyle(int style) override {
+		return style;
+	}
+	void SCI_METHOD FreeSubStyles() override {
+		subStyles.Free();
+	}
+	void SCI_METHOD SetIdentifiers(int style, const char *identifiers) override {
+		subStyles.SetIdentifiers(style, identifiers);
+	}
+	int SCI_METHOD DistanceToSecondaryStyles() override {
+		return 0;
+	}
+	const char *SCI_METHOD GetSubStyleBases() override {
+		return styleSubable;
+	}
+
 	static ILexer5 *LexerFactoryLua() {
 		return new LexerLua();
 	}
@@ -206,6 +244,8 @@ void LexerLua::Lex(Sci_PositionU startPos, Sci_Position length, int initStyle, I
 	const CharacterSet setLuaOperator("*/-+()={}~[];<>,.^%:#&|");
 	const CharacterSet setEscapeSkip("\"'\\");
 
+	const WordClassifier &classifierIdentifiers = subStyles.Classifier(SCE_LUA_IDENTIFIER);
+
 	Sci_Position currentLine = styler.GetLine(startPos);
 	// Initialize long string [[ ... ]] or block comment --[[ ... ]],
 	// if we are inside such a string. Block comment was introduced in Lua 5.0,
@@ -224,6 +264,7 @@ void LexerLua::Lex(Sci_PositionU startPos, Sci_Position length, int initStyle, I
 
 	// results of identifier/keyword matching
 	Sci_Position idenPos = 0;
+	Sci_Position idenStartCharWidth = 0;
 	Sci_Position idenWordPos = 0;
 	int idenStyle = SCE_LUA_IDENTIFIER;
 	bool foundGoto = false;
@@ -280,22 +321,22 @@ void LexerLua::Lex(Sci_PositionU startPos, Sci_Position length, int initStyle, I
 			if (sc.ch == ':' && sc.chPrev == ':') {	// :: <label> :: forward scan
 				sc.Forward();
 				Sci_Position ln = 0;
-				while (IsASpaceOrTab(sc.GetRelative(ln)))	// skip over spaces/tabs
+				while (IsASpaceOrTab(sc.GetRelativeChar(ln)))	// skip over spaces/tabs
 					ln++;
 				const Sci_Position ws1 = ln;
-				if (setWordStart.Contains(sc.GetRelative(ln))) {
-					int c = 0;
+				if (setWordStart.Contains(sc.GetRelativeChar(ln))) {
+					char cLabel = 0;
 					std::string s;
-					while (setWord.Contains(c = sc.GetRelative(ln))) {	// get potential label
-						s.push_back(static_cast<char>(c));
+					while (setWord.Contains(cLabel = sc.GetRelativeChar(ln))) {	// get potential label
+						s.push_back(cLabel);
 						ln++;
 					}
 					const Sci_Position lbl = ln;
 					if (!keywords.InList(s)) {
-						while (IsASpaceOrTab(sc.GetRelative(ln)))	// skip over spaces/tabs
+						while (IsASpaceOrTab(sc.GetRelativeChar(ln)))	// skip over spaces/tabs
 							ln++;
 						const Sci_Position ws2 = ln - lbl;
-						if (sc.GetRelative(ln) == ':' && sc.GetRelative(ln + 1) == ':') {
+						if (sc.GetRelativeChar(ln) == ':' && sc.GetRelativeChar(ln + 1) == ':') {
 							// final :: found, complete valid label construct
 							sc.ChangeState(SCE_LUA_LABEL);
 							if (ws1) {
@@ -324,7 +365,7 @@ void LexerLua::Lex(Sci_PositionU startPos, Sci_Position length, int initStyle, I
 					sc.SetState(SCE_LUA_DEFAULT);
 			}
 		} else if (sc.state == SCE_LUA_IDENTIFIER) {
-			idenPos--;			// commit already-scanned identitier/word parts
+			idenPos -= idenStartCharWidth;			// commit already-scanned identifier/word parts
 			if (idenWordPos > 0) {
 				idenWordPos--;
 				sc.ChangeState(idenStyle);
@@ -405,7 +446,7 @@ void LexerLua::Lex(Sci_PositionU startPos, Sci_Position length, int initStyle, I
 		if (sc.state == SCE_LUA_DEFAULT) {
 			if (IsADigit(sc.ch) || (sc.ch == '.' && IsADigit(sc.chNext))) {
 				sc.SetState(SCE_LUA_NUMBER);
-				if (sc.ch == '0' && toupper(sc.chNext) == 'X') {
+				if (sc.ch == '0' && AnyOf(sc.chNext, 'x', 'X')) {
 					sc.Forward();
 				}
 			} else if (setWordStart.Contains(sc.ch)) {
@@ -414,17 +455,18 @@ void LexerLua::Lex(Sci_PositionU startPos, Sci_Position length, int initStyle, I
 				// set to a word style. The non-matched part is in identifier style.
 				std::string ident;
 				idenPos = 0;
+				idenStartCharWidth = sc.width;
 				idenWordPos = 0;
 				idenStyle = SCE_LUA_IDENTIFIER;
 				foundGoto = false;
-				int cNext = 0;
+				char cNext = 0;
 				do {
-					int c = 0;
+					char cIdent = 0;
 					const Sci_Position idenPosOld = idenPos;
 					std::string identSeg;
-					identSeg += static_cast<char>(sc.GetRelative(idenPos++));
-					while (setWord.Contains(c = sc.GetRelative(idenPos))) {
-						identSeg += static_cast<char>(c);
+					identSeg += sc.GetRelativeChar(idenPos++);
+					while (setWord.Contains(cIdent = sc.GetRelativeChar(idenPos))) {
+						identSeg += cIdent;
 						idenPos++;
 					}
 					if (keywords.InList(identSeg) && (idenPosOld > 0)) {
@@ -450,6 +492,11 @@ void LexerLua::Lex(Sci_PositionU startPos, Sci_Position length, int initStyle, I
 						newStyle = SCE_LUA_WORD7;
 					} else if (keywords8.InList(ident)) {
 						newStyle = SCE_LUA_WORD8;
+					} else {
+						const int subStyle = classifierIdentifiers.ValueFor(ident);
+						if (subStyle >= 0) {
+							newStyle = subStyle;
+						}
 					}
 					if (newStyle != SCE_LUA_IDENTIFIER) {
 						idenStyle = newStyle;
@@ -457,15 +504,15 @@ void LexerLua::Lex(Sci_PositionU startPos, Sci_Position length, int initStyle, I
 					}
 					if (idenStyle == SCE_LUA_WORD)	// keywords cannot mix
 						break;
-					cNext = sc.GetRelative(idenPos + 1);
-					if ((c == '.' || c == ':') && setWordStart.Contains(cNext)) {
-						ident += static_cast<char>(c);
+					cNext = sc.GetRelativeChar(idenPos + 1);
+					if ((cIdent == '.' || cIdent == ':') && setWordStart.Contains(cNext)) {
+						ident += cIdent;
 						idenPos++;
 					} else {
 						cNext = 0;
 					}
 				} while (cNext);
-                if ((idenStyle == SCE_LUA_WORD) && (ident == "goto")) {
+				if ((idenStyle == SCE_LUA_WORD) && (ident == "goto")) {
 					foundGoto = true;
 				}
 				sc.SetState(SCE_LUA_IDENTIFIER);
@@ -530,14 +577,18 @@ void LexerLua::Fold(Sci_PositionU startPos_, Sci_Position length, int initStyle,
 		chNext = styler.SafeGetCharAt(i + 1);
 		const int stylePrev = style;
 		style = styleNext;
-		styleNext = styler.StyleIndexAt(i + 1);
+		if ((i + 1) < lengthDoc) {
+			// Only read styles that have been set, otherwise treat style as continuing
+			styleNext = styler.StyleIndexAt(i + 1);
+		}
 		const bool atEOL = (ch == '\r' && chNext != '\n') || (ch == '\n');
 		if (style == SCE_LUA_WORD) {
 			// Fixed list of folding words: if, do, function, repeat, end, until
 			// Must fix up next line with initial characters if any new words added.
 			if ((style != stylePrev) && AnyOf(ch, 'i', 'd', 'f', 'e', 'r', 'u')) {
+				constexpr Sci_Position maxFoldWord = 9; // "function"sv.length() + 1
 				std::string s;
-				for (Sci_Position j = 0; j < 8; j++) {	// 8 is length of longest: function
+				for (Sci_Position j = 0; j < maxFoldWord; j++) {
 					if (!iswordchar(styler[i + j])) {
 						break;
 					}
