@@ -27,7 +27,7 @@ public:
 	int xOffset;		///< Horizontal scrolled amount in pixels
 	bool trackLineWidth;
 
-	SpecialRepresentations reprs;
+	std::unique_ptr<SpecialRepresentations> reprs;
 	Caret caret;
 	SelectionPosition posDrag;
 	Sci::Position braces[2];
