@@ -1146,12 +1146,12 @@ void SciTEBase::StripTrailingSpaces() {
 	for (SA::Line line = 0; line < maxLines; line++) {
 		const SA::Position lineStart = wEditor.LineStart(line);
 		const SA::Position lineEnd = wEditor.LineEnd(line);
-		SA::Position i = lineEnd;
-		while ((i > lineStart) && IsSpaceOrTab(wEditor.CharacterAt(i-1))) {
-			i--;
+		SA::Position firstSpace = lineEnd;
+		while ((firstSpace > lineStart) && IsSpaceOrTab(wEditor.CharacterAt(firstSpace-1))) {
+			firstSpace--;
 		}
-		if (i < lineEnd) {
-			wEditor.DeleteRange(i, lineEnd-i);
+		if (firstSpace < lineEnd) {
+			wEditor.DeleteRange(firstSpace, lineEnd-firstSpace);
 		}
 	}
 }
