@@ -583,6 +583,8 @@ static const char *propertiesToForward[] = {
 	"lexer.haskell.allow.quotes",
 	"lexer.haskell.cpp",
 	"lexer.haskell.import.safe",
+	"lexer.html.allow.asp",
+	"lexer.html.allow.php",
 	"lexer.html.django",
 	"lexer.html.mako",
 	"lexer.json.allow.comments",
@@ -622,6 +624,8 @@ static const char *propertiesToForward[] = {
 	"lexer.verilog.update.preprocessor",
 	"lexer.visualprolog.backquoted.strings",
 	"lexer.visualprolog.verbatim.strings",
+	"lexer.xml.allow.asp",
+	"lexer.xml.allow.php",
 	"lexer.xml.allow.scripts",
 	"nsis.ignorecase",
 	"nsis.uservars",
@@ -797,7 +801,7 @@ void SciTEBase::ReadProperties() {
 	Lexilla::Load(lexillaPath.empty() ? "." : lexillaPath);
 
 	std::vector<std::string> libraryProperties = Lexilla::LibraryProperties();
-	for (std::string property : libraryProperties) {
+	for (const std::string &property : libraryProperties) {
 		std::string key("lexilla.context.");
 		key += property;
 		std::string value = props.GetExpandedString(key);
