@@ -1,3 +1,5 @@
+#!/usr/bin/env node
+
 // A demonstration program
 #include <stdio.h>
 #if 0 /* */
@@ -28,6 +30,40 @@ b
 
 \
  
+
+// Test preprocessor active branches feature
+
+#if HAVE_COLOUR
+// Active
+#endif
+#if NOT_HAVE_COLOUR
+// Inactive
+#endif
+
+#if FEATURE==2
+// Active
+#endif
+#if FEATURE==3
+// Inactive
+#endif
+
+#if VERSION(1,2)==3
+// Active
+#endif
+#if VERSION(1,2)==4
+// Inactive
+#endif
+
+#undef HAVE_COLOUR
+#if HAVE_COLOUR
+// Inactive
+#endif
+
+#define MULTIPLY(a,b) a*b
+#if MULTIPLY(2,3)==6
+// Active
+#endif
+
 int main() {
 	double x[] = {3.14159,6.02e23,1.6e-19,1.0+1};
 	int y[] = {75,0113,0x4b};
@@ -36,6 +72,9 @@ int main() {
 	// JavaScript regular expression (14) tests
 	let a = /a/;
 	let b = /[a-z]+/gi;
+	/a|b/i.test("baby");
+	// arrow function
+	() => /a|b/i.test("baby");
 
 	// Escape sequence (27) tests
 	printf("\'\"\?\\\a\b\f\n\r\t\v \P");
